@@ -11,8 +11,8 @@ __author__ = 'Martin Pitt'
 __copyright__ = '(c) 2012 Canonical Ltd.'
 
 import copy
-import imp
 import importlib
+import importlib.util
 import os
 import sys
 import time
@@ -44,8 +44,10 @@ def load_module(name: str):
     '''Load a mock template Python module from dbusmock/templates/'''
 
     if os.path.exists(name) and os.path.splitext(name)[1] == '.py':
-        mod = imp.new_module(os.path.splitext(os.path.basename(name))[0])
-        with open(name) as f:
+        spec = importlib.util.spec_from_file_location(os.path.splitext(os.path.basename(name))[0], name)
+        assert spec
+        mod = importlib.util.module_from_spec(spec)
+        with open(name, encoding="UTF-8") as f:
             exec(f.read(), mod.__dict__, mod.__dict__)  # pylint: disable=exec-used
         return mod
 
@@ -114,8 +116,7 @@ def _wrap_in_dbus_variant(value):
         return type(value)(value.conjugate(), variant_level=1)
     if isinstance(value, str):
         return dbus.String(value, variant_level=1)
-    raise dbus.exceptions.DBusException(
-        'could not wrap type {}'.format(type(value)))
+    raise dbus.exceptions.DBusException(f'could not wrap type {type(value)}')
 
 
 class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-attributes
@@ -158,6 +159,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
         self._template: Optional[str] = None
         self._template_parameters: Optional[PropsType] = None
 
+        # pylint: disable=consider-using-with
         self.logfile = open(logfile, 'wb') if logfile else None
         self.is_logfile_owner = True
         self.call_log: List[CallLogType] = []
@@ -176,7 +178,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
         if self.path == '/':
             cond = 'k != \'/\''
         else:
-            cond = 'k.startswith(\'%s/\')' % self.path
+            cond = f'k.startswith(\'{self.path}/\')'
 
         self.AddMethod(OBJECT_MANAGER_IFACE,
                        'GetManagedObjects', '', 'a{oa{sa{sv}}}',
@@ -199,7 +201,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
     def Get(self, interface_name: str, property_name: str) -> Any:
         '''Standard D-Bus API for getting a property value'''
 
-        self.log('Get %s.%s' % (interface_name, property_name))
+        self.log(f'Get {interface_name}.{property_name}')
 
         if not interface_name:
             interface_name = self.interface
@@ -231,9 +233,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
     def Set(self, interface_name: str, property_name: str, value: Any, *_, **__) -> None:
         '''Standard D-Bus API for setting a property value'''
 
-        self.log('Set %s.%s%s' % (interface_name,
-                                  property_name,
-                                  _format_args((value,))))
+        self.log(f'Set {interface_name}.{property_name}{_format_args((value,))}')
 
         try:
             iface_props = self.props[interface_name]
@@ -290,9 +290,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
                              ])
         '''
         if path in objects:
-            raise dbus.exceptions.DBusException(
-                'object %s already exists' % path,
-                name='org.freedesktop.DBus.Mock.NameError')
+            raise dbus.exceptions.DBusException(f'object {path} already exists', name='org.freedesktop.DBus.Mock.NameError')
 
         obj = DBusMockObject(self.bus_name,
                              path,
@@ -320,7 +318,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
             del objects[path]
         except KeyError as e:
             raise dbus.exceptions.DBusException(
-                'object %s does not exist' % path,
+                f'object {path} does not exist',
                 name='org.freedesktop.DBus.Mock.NameError') from e
 
     @dbus.service.method(MOCK_IFACE,
@@ -406,7 +404,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
                                           out_signature=out_sig)(method)
         dbus_method.__name__ = str(name)
         dbus_method._dbus_in_signature = in_sig
-        dbus_method._dbus_args = ['arg%i' % i for i in range(1, n_args + 1)]
+        dbus_method._dbus_args = [f'arg{i}' for i in range(1, n_args + 1)]
 
         # for convenience, add mocked methods on the primary interface as
         # callable methods
@@ -456,9 +454,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
             if not interface:
                 interface = self.interface
             if name not in self.props.get(interface, {}):
-                raise dbus.exceptions.DBusException(
-                    'property %s not found' % name,
-                    name=interface + '.NoSuchProperty')
+                raise dbus.exceptions.DBusException(f'property {name} not found', name=interface + '.NoSuchProperty')
 
             self._set_property(interface, name, value)
             changed_props[name] = _wrap_in_dbus_variant(value)
@@ -481,9 +477,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
         if not interface:
             interface = self.interface
         if name in self.props.get(interface, {}):
-            raise dbus.exceptions.DBusException(
-                'property %s already exists' % name,
-                name=self.interface + '.PropertyExists')
+            raise dbus.exceptions.DBusException(f'property {name} already exists', name=self.interface + '.PropertyExists')
 
         self._set_property(interface, name, value)
 
@@ -526,7 +520,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
         try:
             module = load_module(template)
         except ImportError as e:
-            raise dbus.exceptions.DBusException('Cannot add template %s: %s' % (template, str(e)),
+            raise dbus.exceptions.DBusException(f'Cannot add template {template}: {str(e)}',
                                                 name='org.freedesktop.DBus.Mock.TemplateError')
 
         # If the template specifies this is an ObjectManager, set that up
@@ -583,11 +577,11 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
         m.append(signature=signature, *args)
         args = m.get_args_list()
 
-        fn = lambda self, *args: self.log('emit %s.%s%s' % (interface, name, _format_args(args)))
+        fn = lambda self, *args: self.log(f'emit {interface}.{name}{_format_args(args)}')
         fn.__name__ = str(name)
         dbus_fn = dbus.service.signal(interface)(fn)
         dbus_fn._dbus_signature = signature
-        dbus_fn._dbus_args = ['arg%i' % i for i in range(1, len(args) + 1)]
+        dbus_fn._dbus_args = [f'arg{i}' for i in range(1, len(args) + 1)]
 
         dbus_fn(self, *args)
 
@@ -690,7 +684,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
         else:
             fd = sys.stdout.fileno()
 
-        os.write(fd, ('%.3f %s\n' % (time.time(), msg)).encode('UTF-8'))
+        os.write(fd, f'{time.time():.3f} {msg}\n'.encode('UTF-8'))
 
     @dbus.service.method(dbus.INTROSPECTABLE_IFACE,
                          in_signature='',
@@ -712,24 +706,24 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
 
         mock_interfaces = orig_interfaces.copy()
         for iface, methods in self.methods.items():
-            for method in methods:
-                mock_interfaces.setdefault(iface, {})[method] = self.methods[iface][method][3]
+            for method, impl in methods.items():
+                mock_interfaces.setdefault(iface, {})[method] = impl[3]
         self._dbus_class_table[cls] = mock_interfaces
 
         xml = dbus.service.Object.Introspect(self, object_path, connection)
 
         tree = ElementTree.fromstring(xml)
 
-        for name in self.props:
+        for name, name_props in self.props.items():
             # We might have properties for new interfaces we don't know about
             # yet. Try to find an existing <interface> node named after our
             # interface to append to, and create one if we can't.
-            interface = tree.find(".//interface[@name='%s']" % name)
+            interface = tree.find(f".//interface[@name='{name}']")
             if interface is None:
                 interface = ElementTree.Element("interface", {"name": name})
                 tree.append(interface)
 
-            for prop, val in self.props[name].items():
+            for prop, val in name_props.items():
                 if val is None:
                     # can't guess type from None, skip
                     continue
